from typing import Optional

from pydantic import BaseModel, ConfigDict


class EmailAuthenticationForm(BaseModel):
    """The body of the form for email authentication."""

    callback_url: Optional[str] = None

    email: str

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.subscription_tier_price import SubscriptionTierPrice
from ..models.uuid import Uuid
from ..models.zoo_product_subscriptions import ZooProductSubscriptions


class CustomerBalance(BaseModel):
    """A balance for a customer.

    This holds information about the financial balance for the customer."""

    created_at: datetime.datetime

    id: Uuid

    map_id: Uuid

    modeling_app_enterprise_price: Optional[SubscriptionTierPrice] = None

    monthly_api_credits_remaining: int

    monthly_api_credits_remaining_monetary_value: float

    stable_api_credits_remaining: int

    stable_api_credits_remaining_monetary_value: float

    subscription_details: Optional[ZooProductSubscriptions] = None

    subscription_id: Optional[str] = None

    total_due: float

    updated_at: datetime.datetime

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class UpdateShortlinkRequest(BaseModel):
    """Request to update a shortlink."""

    password: Optional[str] = None

    restrict_to_org: bool

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class ModelingAppShareLinks(str, Enum):
    """The options for sharable links through the modeling app."""  # noqa: E501

    """# Public. """  # noqa: E501
    PUBLIC = "public"
    """# Password protected. """  # noqa: E501
    PASSWORD_PROTECTED = "password_protected"
    """# Organization only. Links can be made only available to members of the organization. """  # noqa: E501
    ORGANIZATION_ONLY = "organization_only"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Dict, List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.currency import Currency
from ..models.discount import Discount
from ..models.invoice_line_item import InvoiceLineItem
from ..models.invoice_status import InvoiceStatus


class Invoice(BaseModel):
    """An invoice."""

    amount_due: float = 0.0

    amount_paid: float = 0.0

    amount_remaining: float = 0.0

    attempt_count: int = 0

    attempted: bool = False

    created_at: datetime.datetime

    currency: Currency = "usd"  # type: ignore

    customer_email: Optional[str] = None

    customer_id: Optional[str] = None

    default_payment_method: Optional[str] = None

    description: Optional[str] = None

    discounts: Optional[List[Discount]] = None

    id: Optional[str] = None

    lines: Optional[List[InvoiceLineItem]] = None

    metadata: Dict[str, str] = {}

    number: Optional[str] = None

    paid: bool = False

    pdf: Optional[str] = None

    receipt_number: Optional[str] = None

    statement_descriptor: Optional[str] = None

    status: Optional[InvoiceStatus] = None

    subtotal: float = 0.0

    tax: float = 0.0

    total: float = 0.0

    url: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict


class AddHoleFromOffset(BaseModel):
    """The response from the `AddHoleFromOffset` command."""

    entity_ids: List[str]

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class Error(BaseModel):
    """Error information from a response."""

    error_code: Optional[str] = None

    message: str

    request_id: str

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.user_org_role import UserOrgRole


class AddOrgMember(BaseModel):
    """Data for adding a member to an org."""

    email: str

    role: UserOrgRole

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.o_auth2_grant_type import OAuth2GrantType


class DeviceAccessTokenRequestForm(BaseModel):
    """The form for a device access token request."""

    client_id: str

    device_code: str

    grant_type: OAuth2GrantType

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class SourcePosition(BaseModel):
    """A position in the source code."""

    column: int

    line: int

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.add_hole_from_offset import AddHoleFromOffset
from ..models.boolean_intersection import BooleanIntersection
from ..models.boolean_subtract import BooleanSubtract
from ..models.boolean_union import BooleanUnion
from ..models.camera_drag_end import CameraDragEnd
from ..models.camera_drag_move import CameraDragMove
from ..models.camera_drag_start import CameraDragStart
from ..models.center_of_mass import CenterOfMass
from ..models.close_path import ClosePath
from ..models.curve_get_control_points import CurveGetControlPoints
from ..models.curve_get_end_points import CurveGetEndPoints
from ..models.curve_get_type import CurveGetType
from ..models.curve_set_constraint import CurveSetConstraint
from ..models.default_camera_center_to_scene import DefaultCameraCenterToScene
from ..models.default_camera_center_to_selection import DefaultCameraCenterToSelection
from ..models.default_camera_focus_on import DefaultCameraFocusOn
from ..models.default_camera_get_settings import DefaultCameraGetSettings
from ..models.default_camera_get_view import DefaultCameraGetView
from ..models.default_camera_look_at import DefaultCameraLookAt
from ..models.default_camera_perspective_settings import (
    DefaultCameraPerspectiveSettings,
)
from ..models.default_camera_set_orthographic import DefaultCameraSetOrthographic
from ..models.default_camera_set_perspective import DefaultCameraSetPerspective
from ..models.default_camera_set_view import DefaultCameraSetView
from ..models.default_camera_zoom import DefaultCameraZoom
from ..models.density import Density
from ..models.disable_dry_run import DisableDryRun
from ..models.edge_lines_visible import EdgeLinesVisible
from ..models.enable_dry_run import EnableDryRun
from ..models.enable_sketch_mode import EnableSketchMode
from ..models.engine_util_evaluate_path import EngineUtilEvaluatePath
from ..models.entity_circular_pattern import EntityCircularPattern
from ..models.entity_clone import EntityClone
from ..models.entity_fade import EntityFade
from ..models.entity_get_all_child_uuids import EntityGetAllChildUuids
from ..models.entity_get_child_uuid import EntityGetChildUuid
from ..models.entity_get_distance import EntityGetDistance
from ..models.entity_get_num_children import EntityGetNumChildren
from ..models.entity_get_parent_id import EntityGetParentId
from ..models.entity_get_sketch_paths import EntityGetSketchPaths
from ..models.entity_linear_pattern import EntityLinearPattern
from ..models.entity_linear_pattern_transform import EntityLinearPatternTransform
from ..models.entity_make_helix import EntityMakeHelix
from ..models.entity_make_helix_from_edge import EntityMakeHelixFromEdge
from ..models.entity_make_helix_from_params import EntityMakeHelixFromParams
from ..models.entity_mirror import EntityMirror
from ..models.entity_mirror_across_edge import EntityMirrorAcrossEdge
from ..models.entity_set_opacity import EntitySetOpacity
from ..models.export import Export
from ..models.export2d import Export2d
from ..models.export3d import Export3d
from ..models.extend_path import ExtendPath
from ..models.extrude import Extrude
from ..models.extrusion_face_info import ExtrusionFaceInfo
from ..models.face_get_center import FaceGetCenter
from ..models.face_get_gradient import FaceGetGradient
from ..models.face_get_position import FaceGetPosition
from ..models.face_is_planar import FaceIsPlanar
from ..models.get_entity_type import GetEntityType
from ..models.get_num_objects import GetNumObjects
from ..models.get_sketch_mode_plane import GetSketchModePlane
from ..models.handle_mouse_drag_end import HandleMouseDragEnd
from ..models.handle_mouse_drag_move import HandleMouseDragMove
from ..models.handle_mouse_drag_start import HandleMouseDragStart
from ..models.highlight_set_entities import HighlightSetEntities
from ..models.highlight_set_entity import HighlightSetEntity
from ..models.import_files import ImportFiles
from ..models.imported_geometry import ImportedGeometry
from ..models.loft import Loft
from ..models.make_axes_gizmo import MakeAxesGizmo
from ..models.make_offset_path import MakeOffsetPath
from ..models.make_plane import MakePlane
from ..models.mass import Mass
from ..models.mouse_click import MouseClick
from ..models.mouse_move import MouseMove
from ..models.move_path_pen import MovePathPen
from ..models.new_annotation import NewAnnotation
from ..models.object_bring_to_front import ObjectBringToFront
from ..models.object_set_material_params_pbr import ObjectSetMaterialParamsPbr
from ..models.object_visible import ObjectVisible
from ..models.orient_to_face import OrientToFace
from ..models.path_get_curve_uuid import PathGetCurveUuid
from ..models.path_get_curve_uuids_for_vertices import PathGetCurveUuidsForVertices
from ..models.path_get_info import PathGetInfo
from ..models.path_get_sketch_target_uuid import PathGetSketchTargetUuid
from ..models.path_get_vertex_uuids import PathGetVertexUuids
from ..models.path_segment_info import PathSegmentInfo
from ..models.plane_intersect_and_project import PlaneIntersectAndProject
from ..models.plane_set_color import PlaneSetColor
from ..models.project_entity_to_plane import ProjectEntityToPlane
from ..models.project_points_to_plane import ProjectPointsToPlane
from ..models.reconfigure_stream import ReconfigureStream
from ..models.remove_scene_objects import RemoveSceneObjects
from ..models.revolve import Revolve
from ..models.revolve_about_edge import RevolveAboutEdge
from ..models.scene_clear_all import SceneClearAll
from ..models.select_add import SelectAdd
from ..models.select_clear import SelectClear
from ..models.select_get import SelectGet
from ..models.select_remove import SelectRemove
from ..models.select_replace import SelectReplace
from ..models.select_with_point import SelectWithPoint
from ..models.send_object import SendObject
from ..models.set_background_color import SetBackgroundColor
from ..models.set_current_tool_properties import SetCurrentToolProperties
from ..models.set_default_system_properties import SetDefaultSystemProperties
from ..models.set_grid_reference_plane import SetGridReferencePlane
from ..models.set_object_transform import SetObjectTransform
from ..models.set_scene_units import SetSceneUnits
from ..models.set_selection_filter import SetSelectionFilter
from ..models.set_selection_type import SetSelectionType
from ..models.set_tool import SetTool
from ..models.sketch_mode_disable import SketchModeDisable
from ..models.solid2d_add_hole import Solid2dAddHole
from ..models.solid3d_fillet_edge import Solid3dFilletEdge
from ..models.solid3d_get_all_edge_faces import Solid3dGetAllEdgeFaces
from ..models.solid3d_get_all_opposite_edges import Solid3dGetAllOppositeEdges
from ..models.solid3d_get_common_edge import Solid3dGetCommonEdge
from ..models.solid3d_get_extrusion_face_info import Solid3dGetExtrusionFaceInfo
from ..models.solid3d_get_next_adjacent_edge import Solid3dGetNextAdjacentEdge
from ..models.solid3d_get_opposite_edge import Solid3dGetOppositeEdge
from ..models.solid3d_get_prev_adjacent_edge import Solid3dGetPrevAdjacentEdge
from ..models.solid3d_shell_face import Solid3dShellFace
from ..models.start_path import StartPath
from ..models.surface_area import SurfaceArea
from ..models.sweep import Sweep
from ..models.take_snapshot import TakeSnapshot
from ..models.update_annotation import UpdateAnnotation
from ..models.view_isometric import ViewIsometric
from ..models.volume import Volume
from ..models.zoom_to_fit import ZoomToFit


class OptionEmpty(BaseModel):
    """An empty response, used for any command that does not explicitly have a response defined here."""

    type: Literal["empty"] = "empty"

    model_config = ConfigDict(protected_namespaces=())


class OptionEngineUtilEvaluatePath(BaseModel):
    """"""

    data: EngineUtilEvaluatePath

    type: Literal["engine_util_evaluate_path"] = "engine_util_evaluate_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionStartPath(BaseModel):
    """"""

    data: StartPath

    type: Literal["start_path"] = "start_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionMovePathPen(BaseModel):
    """"""

    data: MovePathPen

    type: Literal["move_path_pen"] = "move_path_pen"

    model_config = ConfigDict(protected_namespaces=())


class OptionExtendPath(BaseModel):
    """"""

    data: ExtendPath

    type: Literal["extend_path"] = "extend_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionExtrude(BaseModel):
    """"""

    data: Extrude

    type: Literal["extrude"] = "extrude"

    model_config = ConfigDict(protected_namespaces=())


class OptionSweep(BaseModel):
    """"""

    data: Sweep

    type: Literal["sweep"] = "sweep"

    model_config = ConfigDict(protected_namespaces=())


class OptionRevolve(BaseModel):
    """"""

    data: Revolve

    type: Literal["revolve"] = "revolve"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DShellFace(BaseModel):
    """"""

    data: Solid3dShellFace

    type: Literal["solid3d_shell_face"] = "solid3d_shell_face"

    model_config = ConfigDict(protected_namespaces=())


class OptionRevolveAboutEdge(BaseModel):
    """"""

    data: RevolveAboutEdge

    type: Literal["revolve_about_edge"] = "revolve_about_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionCameraDragStart(BaseModel):
    """"""

    data: CameraDragStart

    type: Literal["camera_drag_start"] = "camera_drag_start"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraLookAt(BaseModel):
    """"""

    data: DefaultCameraLookAt

    type: Literal["default_camera_look_at"] = "default_camera_look_at"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraPerspectiveSettings(BaseModel):
    """"""

    data: DefaultCameraPerspectiveSettings

    type: Literal["default_camera_perspective_settings"] = (
        "default_camera_perspective_settings"
    )

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectAdd(BaseModel):
    """"""

    data: SelectAdd

    type: Literal["select_add"] = "select_add"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectRemove(BaseModel):
    """"""

    data: SelectRemove

    type: Literal["select_remove"] = "select_remove"

    model_config = ConfigDict(protected_namespaces=())


class OptionSceneClearAll(BaseModel):
    """"""

    data: SceneClearAll

    type: Literal["scene_clear_all"] = "scene_clear_all"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectReplace(BaseModel):
    """"""

    data: SelectReplace

    type: Literal["select_replace"] = "select_replace"

    model_config = ConfigDict(protected_namespaces=())


class OptionHighlightSetEntities(BaseModel):
    """"""

    data: HighlightSetEntities

    type: Literal["highlight_set_entities"] = "highlight_set_entities"

    model_config = ConfigDict(protected_namespaces=())


class OptionNewAnnotation(BaseModel):
    """"""

    data: NewAnnotation

    type: Literal["new_annotation"] = "new_annotation"

    model_config = ConfigDict(protected_namespaces=())


class OptionUpdateAnnotation(BaseModel):
    """"""

    data: UpdateAnnotation

    type: Literal["update_annotation"] = "update_annotation"

    model_config = ConfigDict(protected_namespaces=())


class OptionEdgeLinesVisible(BaseModel):
    """"""

    data: EdgeLinesVisible

    type: Literal["edge_lines_visible"] = "edge_lines_visible"

    model_config = ConfigDict(protected_namespaces=())


class OptionObjectVisible(BaseModel):
    """"""

    data: ObjectVisible

    type: Literal["object_visible"] = "object_visible"

    model_config = ConfigDict(protected_namespaces=())


class OptionObjectBringToFront(BaseModel):
    """"""

    data: ObjectBringToFront

    type: Literal["object_bring_to_front"] = "object_bring_to_front"

    model_config = ConfigDict(protected_namespaces=())


class OptionObjectSetMaterialParamsPbr(BaseModel):
    """"""

    data: ObjectSetMaterialParamsPbr

    type: Literal["object_set_material_params_pbr"] = "object_set_material_params_pbr"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid2DAddHole(BaseModel):
    """"""

    data: Solid2dAddHole

    type: Literal["solid2d_add_hole"] = "solid2d_add_hole"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DFilletEdge(BaseModel):
    """"""

    data: Solid3dFilletEdge

    type: Literal["solid3d_fillet_edge"] = "solid3d_fillet_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSendObject(BaseModel):
    """"""

    data: SendObject

    type: Literal["send_object"] = "send_object"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntitySetOpacity(BaseModel):
    """"""

    data: EntitySetOpacity

    type: Literal["entity_set_opacity"] = "entity_set_opacity"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityFade(BaseModel):
    """"""

    data: EntityFade

    type: Literal["entity_fade"] = "entity_fade"

    model_config = ConfigDict(protected_namespaces=())


class OptionMakePlane(BaseModel):
    """"""

    data: MakePlane

    type: Literal["make_plane"] = "make_plane"

    model_config = ConfigDict(protected_namespaces=())


class OptionPlaneSetColor(BaseModel):
    """"""

    data: PlaneSetColor

    type: Literal["plane_set_color"] = "plane_set_color"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetTool(BaseModel):
    """"""

    data: SetTool

    type: Literal["set_tool"] = "set_tool"

    model_config = ConfigDict(protected_namespaces=())


class OptionMouseMove(BaseModel):
    """"""

    data: MouseMove

    type: Literal["mouse_move"] = "mouse_move"

    model_config = ConfigDict(protected_namespaces=())


class OptionSketchModeDisable(BaseModel):
    """"""

    data: SketchModeDisable

    type: Literal["sketch_mode_disable"] = "sketch_mode_disable"

    model_config = ConfigDict(protected_namespaces=())


class OptionEnableDryRun(BaseModel):
    """"""

    data: EnableDryRun

    type: Literal["enable_dry_run"] = "enable_dry_run"

    model_config = ConfigDict(protected_namespaces=())


class OptionDisableDryRun(BaseModel):
    """"""

    data: DisableDryRun

    type: Literal["disable_dry_run"] = "disable_dry_run"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveSetConstraint(BaseModel):
    """"""

    data: CurveSetConstraint

    type: Literal["curve_set_constraint"] = "curve_set_constraint"

    model_config = ConfigDict(protected_namespaces=())


class OptionEnableSketchMode(BaseModel):
    """"""

    data: EnableSketchMode

    type: Literal["enable_sketch_mode"] = "enable_sketch_mode"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetBackgroundColor(BaseModel):
    """"""

    data: SetBackgroundColor

    type: Literal["set_background_color"] = "set_background_color"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetCurrentToolProperties(BaseModel):
    """"""

    data: SetCurrentToolProperties

    type: Literal["set_current_tool_properties"] = "set_current_tool_properties"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetDefaultSystemProperties(BaseModel):
    """"""

    data: SetDefaultSystemProperties

    type: Literal["set_default_system_properties"] = "set_default_system_properties"

    model_config = ConfigDict(protected_namespaces=())


class OptionMakeAxesGizmo(BaseModel):
    """"""

    data: MakeAxesGizmo

    type: Literal["make_axes_gizmo"] = "make_axes_gizmo"

    model_config = ConfigDict(protected_namespaces=())


class OptionHandleMouseDragStart(BaseModel):
    """"""

    data: HandleMouseDragStart

    type: Literal["handle_mouse_drag_start"] = "handle_mouse_drag_start"

    model_config = ConfigDict(protected_namespaces=())


class OptionHandleMouseDragMove(BaseModel):
    """"""

    data: HandleMouseDragMove

    type: Literal["handle_mouse_drag_move"] = "handle_mouse_drag_move"

    model_config = ConfigDict(protected_namespaces=())


class OptionHandleMouseDragEnd(BaseModel):
    """"""

    data: HandleMouseDragEnd

    type: Literal["handle_mouse_drag_end"] = "handle_mouse_drag_end"

    model_config = ConfigDict(protected_namespaces=())


class OptionRemoveSceneObjects(BaseModel):
    """"""

    data: RemoveSceneObjects

    type: Literal["remove_scene_objects"] = "remove_scene_objects"

    model_config = ConfigDict(protected_namespaces=())


class OptionReconfigureStream(BaseModel):
    """"""

    data: ReconfigureStream

    type: Literal["reconfigure_stream"] = "reconfigure_stream"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetSceneUnits(BaseModel):
    """"""

    data: SetSceneUnits

    type: Literal["set_scene_units"] = "set_scene_units"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetSelectionType(BaseModel):
    """"""

    data: SetSelectionType

    type: Literal["set_selection_type"] = "set_selection_type"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetSelectionFilter(BaseModel):
    """"""

    data: SetSelectionFilter

    type: Literal["set_selection_filter"] = "set_selection_filter"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraSetOrthographic(BaseModel):
    """"""

    data: DefaultCameraSetOrthographic

    type: Literal["default_camera_set_orthographic"] = "default_camera_set_orthographic"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraSetPerspective(BaseModel):
    """"""

    data: DefaultCameraSetPerspective

    type: Literal["default_camera_set_perspective"] = "default_camera_set_perspective"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraCenterToSelection(BaseModel):
    """"""

    data: DefaultCameraCenterToSelection

    type: Literal["default_camera_center_to_selection"] = (
        "default_camera_center_to_selection"
    )

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraCenterToScene(BaseModel):
    """"""

    data: DefaultCameraCenterToScene

    type: Literal["default_camera_center_to_scene"] = "default_camera_center_to_scene"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectClear(BaseModel):
    """"""

    data: SelectClear

    type: Literal["select_clear"] = "select_clear"

    model_config = ConfigDict(protected_namespaces=())


class OptionExport2D(BaseModel):
    """"""

    data: Export2d

    type: Literal["export2d"] = "export2d"

    model_config = ConfigDict(protected_namespaces=())


class OptionExport3D(BaseModel):
    """"""

    data: Export3d

    type: Literal["export3d"] = "export3d"

    model_config = ConfigDict(protected_namespaces=())


class OptionExport(BaseModel):
    """"""

    data: Export

    type: Literal["export"] = "export"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectWithPoint(BaseModel):
    """"""

    data: SelectWithPoint

    type: Literal["select_with_point"] = "select_with_point"

    model_config = ConfigDict(protected_namespaces=())


class OptionHighlightSetEntity(BaseModel):
    """"""

    data: HighlightSetEntity

    type: Literal["highlight_set_entity"] = "highlight_set_entity"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetChildUuid(BaseModel):
    """"""

    data: EntityGetChildUuid

    type: Literal["entity_get_child_uuid"] = "entity_get_child_uuid"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetNumChildren(BaseModel):
    """"""

    data: EntityGetNumChildren

    type: Literal["entity_get_num_children"] = "entity_get_num_children"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetParentId(BaseModel):
    """"""

    data: EntityGetParentId

    type: Literal["entity_get_parent_id"] = "entity_get_parent_id"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetAllChildUuids(BaseModel):
    """"""

    data: EntityGetAllChildUuids

    type: Literal["entity_get_all_child_uuids"] = "entity_get_all_child_uuids"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetSketchPaths(BaseModel):
    """"""

    data: EntityGetSketchPaths

    type: Literal["entity_get_sketch_paths"] = "entity_get_sketch_paths"

    model_config = ConfigDict(protected_namespaces=())


class OptionLoft(BaseModel):
    """"""

    data: Loft

    type: Literal["loft"] = "loft"

    model_config = ConfigDict(protected_namespaces=())


class OptionClosePath(BaseModel):
    """"""

    data: ClosePath

    type: Literal["close_path"] = "close_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionCameraDragMove(BaseModel):
    """"""

    data: CameraDragMove

    type: Literal["camera_drag_move"] = "camera_drag_move"

    model_config = ConfigDict(protected_namespaces=())


class OptionCameraDragEnd(BaseModel):
    """"""

    data: CameraDragEnd

    type: Literal["camera_drag_end"] = "camera_drag_end"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraGetSettings(BaseModel):
    """"""

    data: DefaultCameraGetSettings

    type: Literal["default_camera_get_settings"] = "default_camera_get_settings"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraGetView(BaseModel):
    """"""

    data: DefaultCameraGetView

    type: Literal["default_camera_get_view"] = "default_camera_get_view"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraSetView(BaseModel):
    """"""

    data: DefaultCameraSetView

    type: Literal["default_camera_set_view"] = "default_camera_set_view"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraZoom(BaseModel):
    """"""

    data: DefaultCameraZoom

    type: Literal["default_camera_zoom"] = "default_camera_zoom"

    model_config = ConfigDict(protected_namespaces=())


class OptionZoomToFit(BaseModel):
    """"""

    data: ZoomToFit

    type: Literal["zoom_to_fit"] = "zoom_to_fit"

    model_config = ConfigDict(protected_namespaces=())


class OptionOrientToFace(BaseModel):
    """"""

    data: OrientToFace

    type: Literal["orient_to_face"] = "orient_to_face"

    model_config = ConfigDict(protected_namespaces=())


class OptionViewIsometric(BaseModel):
    """"""

    data: ViewIsometric

    type: Literal["view_isometric"] = "view_isometric"

    model_config = ConfigDict(protected_namespaces=())


class OptionGetNumObjects(BaseModel):
    """"""

    data: GetNumObjects

    type: Literal["get_num_objects"] = "get_num_objects"

    model_config = ConfigDict(protected_namespaces=())


class OptionMakeOffsetPath(BaseModel):
    """"""

    data: MakeOffsetPath

    type: Literal["make_offset_path"] = "make_offset_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetObjectTransform(BaseModel):
    """"""

    data: SetObjectTransform

    type: Literal["set_object_transform"] = "set_object_transform"

    model_config = ConfigDict(protected_namespaces=())


class OptionAddHoleFromOffset(BaseModel):
    """"""

    data: AddHoleFromOffset

    type: Literal["add_hole_from_offset"] = "add_hole_from_offset"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraFocusOn(BaseModel):
    """"""

    data: DefaultCameraFocusOn

    type: Literal["default_camera_focus_on"] = "default_camera_focus_on"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectGet(BaseModel):
    """"""

    data: SelectGet

    type: Literal["select_get"] = "select_get"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetAllEdgeFaces(BaseModel):
    """"""

    data: Solid3dGetAllEdgeFaces

    type: Literal["solid3d_get_all_edge_faces"] = "solid3d_get_all_edge_faces"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetAllOppositeEdges(BaseModel):
    """"""

    data: Solid3dGetAllOppositeEdges

    type: Literal["solid3d_get_all_opposite_edges"] = "solid3d_get_all_opposite_edges"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetOppositeEdge(BaseModel):
    """"""

    data: Solid3dGetOppositeEdge

    type: Literal["solid3d_get_opposite_edge"] = "solid3d_get_opposite_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetNextAdjacentEdge(BaseModel):
    """"""

    data: Solid3dGetNextAdjacentEdge

    type: Literal["solid3d_get_next_adjacent_edge"] = "solid3d_get_next_adjacent_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetPrevAdjacentEdge(BaseModel):
    """"""

    data: Solid3dGetPrevAdjacentEdge

    type: Literal["solid3d_get_prev_adjacent_edge"] = "solid3d_get_prev_adjacent_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetCommonEdge(BaseModel):
    """"""

    data: Solid3dGetCommonEdge

    type: Literal["solid3d_get_common_edge"] = "solid3d_get_common_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionGetEntityType(BaseModel):
    """"""

    data: GetEntityType

    type: Literal["get_entity_type"] = "get_entity_type"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveGetControlPoints(BaseModel):
    """"""

    data: CurveGetControlPoints

    type: Literal["curve_get_control_points"] = "curve_get_control_points"

    model_config = ConfigDict(protected_namespaces=())


class OptionProjectEntityToPlane(BaseModel):
    """"""

    data: ProjectEntityToPlane

    type: Literal["project_entity_to_plane"] = "project_entity_to_plane"

    model_config = ConfigDict(protected_namespaces=())


class OptionProjectPointsToPlane(BaseModel):
    """"""

    data: ProjectPointsToPlane

    type: Literal["project_points_to_plane"] = "project_points_to_plane"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveGetType(BaseModel):
    """"""

    data: CurveGetType

    type: Literal["curve_get_type"] = "curve_get_type"

    model_config = ConfigDict(protected_namespaces=())


class OptionMouseClick(BaseModel):
    """"""

    data: MouseClick

    type: Literal["mouse_click"] = "mouse_click"

    model_config = ConfigDict(protected_namespaces=())


class OptionTakeSnapshot(BaseModel):
    """"""

    data: TakeSnapshot

    type: Literal["take_snapshot"] = "take_snapshot"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetInfo(BaseModel):
    """"""

    data: PathGetInfo

    type: Literal["path_get_info"] = "path_get_info"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathSegmentInfo(BaseModel):
    """"""

    data: PathSegmentInfo

    type: Literal["path_segment_info"] = "path_segment_info"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetCurveUuidsForVertices(BaseModel):
    """"""

    data: PathGetCurveUuidsForVertices

    type: Literal["path_get_curve_uuids_for_vertices"] = (
        "path_get_curve_uuids_for_vertices"
    )

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetCurveUuid(BaseModel):
    """"""

    data: PathGetCurveUuid

    type: Literal["path_get_curve_uuid"] = "path_get_curve_uuid"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetVertexUuids(BaseModel):
    """"""

    data: PathGetVertexUuids

    type: Literal["path_get_vertex_uuids"] = "path_get_vertex_uuids"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetSketchTargetUuid(BaseModel):
    """"""

    data: PathGetSketchTargetUuid

    type: Literal["path_get_sketch_target_uuid"] = "path_get_sketch_target_uuid"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveGetEndPoints(BaseModel):
    """"""

    data: CurveGetEndPoints

    type: Literal["curve_get_end_points"] = "curve_get_end_points"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceIsPlanar(BaseModel):
    """"""

    data: FaceIsPlanar

    type: Literal["face_is_planar"] = "face_is_planar"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceGetPosition(BaseModel):
    """"""

    data: FaceGetPosition

    type: Literal["face_get_position"] = "face_get_position"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceGetCenter(BaseModel):
    """"""

    data: FaceGetCenter

    type: Literal["face_get_center"] = "face_get_center"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceGetGradient(BaseModel):
    """"""

    data: FaceGetGradient

    type: Literal["face_get_gradient"] = "face_get_gradient"

    model_config = ConfigDict(protected_namespaces=())


class OptionPlaneIntersectAndProject(BaseModel):
    """"""

    data: PlaneIntersectAndProject

    type: Literal["plane_intersect_and_project"] = "plane_intersect_and_project"

    model_config = ConfigDict(protected_namespaces=())


class OptionImportFiles(BaseModel):
    """"""

    data: ImportFiles

    type: Literal["import_files"] = "import_files"

    model_config = ConfigDict(protected_namespaces=())


class OptionImportedGeometry(BaseModel):
    """"""

    data: ImportedGeometry

    type: Literal["imported_geometry"] = "imported_geometry"

    model_config = ConfigDict(protected_namespaces=())


class OptionMass(BaseModel):
    """"""

    data: Mass

    type: Literal["mass"] = "mass"

    model_config = ConfigDict(protected_namespaces=())


class OptionVolume(BaseModel):
    """"""

    data: Volume

    type: Literal["volume"] = "volume"

    model_config = ConfigDict(protected_namespaces=())


class OptionDensity(BaseModel):
    """"""

    data: Density

    type: Literal["density"] = "density"

    model_config = ConfigDict(protected_namespaces=())


class OptionSurfaceArea(BaseModel):
    """"""

    data: SurfaceArea

    type: Literal["surface_area"] = "surface_area"

    model_config = ConfigDict(protected_namespaces=())


class OptionCenterOfMass(BaseModel):
    """"""

    data: CenterOfMass

    type: Literal["center_of_mass"] = "center_of_mass"

    model_config = ConfigDict(protected_namespaces=())


class OptionGetSketchModePlane(BaseModel):
    """"""

    data: GetSketchModePlane

    type: Literal["get_sketch_mode_plane"] = "get_sketch_mode_plane"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetDistance(BaseModel):
    """"""

    data: EntityGetDistance

    type: Literal["entity_get_distance"] = "entity_get_distance"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityClone(BaseModel):
    """"""

    data: EntityClone

    type: Literal["entity_clone"] = "entity_clone"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityLinearPatternTransform(BaseModel):
    """"""

    data: EntityLinearPatternTransform

    type: Literal["entity_linear_pattern_transform"] = "entity_linear_pattern_transform"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityLinearPattern(BaseModel):
    """"""

    data: EntityLinearPattern

    type: Literal["entity_linear_pattern"] = "entity_linear_pattern"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityCircularPattern(BaseModel):
    """"""

    data: EntityCircularPattern

    type: Literal["entity_circular_pattern"] = "entity_circular_pattern"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMirror(BaseModel):
    """"""

    data: EntityMirror

    type: Literal["entity_mirror"] = "entity_mirror"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMirrorAcrossEdge(BaseModel):
    """"""

    data: EntityMirrorAcrossEdge

    type: Literal["entity_mirror_across_edge"] = "entity_mirror_across_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMakeHelix(BaseModel):
    """"""

    data: EntityMakeHelix

    type: Literal["entity_make_helix"] = "entity_make_helix"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMakeHelixFromParams(BaseModel):
    """"""

    data: EntityMakeHelixFromParams

    type: Literal["entity_make_helix_from_params"] = "entity_make_helix_from_params"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMakeHelixFromEdge(BaseModel):
    """"""

    data: EntityMakeHelixFromEdge

    type: Literal["entity_make_helix_from_edge"] = "entity_make_helix_from_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetExtrusionFaceInfo(BaseModel):
    """"""

    data: Solid3dGetExtrusionFaceInfo

    type: Literal["solid3d_get_extrusion_face_info"] = "solid3d_get_extrusion_face_info"

    model_config = ConfigDict(protected_namespaces=())


class OptionExtrusionFaceInfo(BaseModel):
    """"""

    data: ExtrusionFaceInfo

    type: Literal["extrusion_face_info"] = "extrusion_face_info"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetGridReferencePlane(BaseModel):
    """"""

    data: SetGridReferencePlane

    type: Literal["set_grid_reference_plane"] = "set_grid_reference_plane"

    model_config = ConfigDict(protected_namespaces=())


class OptionBooleanUnion(BaseModel):
    """"""

    data: BooleanUnion

    type: Literal["boolean_union"] = "boolean_union"

    model_config = ConfigDict(protected_namespaces=())


class OptionBooleanIntersection(BaseModel):
    """"""

    data: BooleanIntersection

    type: Literal["boolean_intersection"] = "boolean_intersection"

    model_config = ConfigDict(protected_namespaces=())


class OptionBooleanSubtract(BaseModel):
    """"""

    data: BooleanSubtract

    type: Literal["boolean_subtract"] = "boolean_subtract"

    model_config = ConfigDict(protected_namespaces=())


OkModelingCmdResponse = RootModel[
    Annotated[
        Union[
            OptionEmpty,
            OptionEngineUtilEvaluatePath,
            OptionStartPath,
            OptionMovePathPen,
            OptionExtendPath,
            OptionExtrude,
            OptionSweep,
            OptionRevolve,
            OptionSolid3DShellFace,
            OptionRevolveAboutEdge,
            OptionCameraDragStart,
            OptionDefaultCameraLookAt,
            OptionDefaultCameraPerspectiveSettings,
            OptionSelectAdd,
            OptionSelectRemove,
            OptionSceneClearAll,
            OptionSelectReplace,
            OptionHighlightSetEntities,
            OptionNewAnnotation,
            OptionUpdateAnnotation,
            OptionEdgeLinesVisible,
            OptionObjectVisible,
            OptionObjectBringToFront,
            OptionObjectSetMaterialParamsPbr,
            OptionSolid2DAddHole,
            OptionSolid3DFilletEdge,
            OptionSendObject,
            OptionEntitySetOpacity,
            OptionEntityFade,
            OptionMakePlane,
            OptionPlaneSetColor,
            OptionSetTool,
            OptionMouseMove,
            OptionSketchModeDisable,
            OptionEnableDryRun,
            OptionDisableDryRun,
            OptionCurveSetConstraint,
            OptionEnableSketchMode,
            OptionSetBackgroundColor,
            OptionSetCurrentToolProperties,
            OptionSetDefaultSystemProperties,
            OptionMakeAxesGizmo,
            OptionHandleMouseDragStart,
            OptionHandleMouseDragMove,
            OptionHandleMouseDragEnd,
            OptionRemoveSceneObjects,
            OptionReconfigureStream,
            OptionSetSceneUnits,
            OptionSetSelectionType,
            OptionSetSelectionFilter,
            OptionDefaultCameraSetOrthographic,
            OptionDefaultCameraSetPerspective,
            OptionDefaultCameraCenterToSelection,
            OptionDefaultCameraCenterToScene,
            OptionSelectClear,
            OptionExport2D,
            OptionExport3D,
            OptionExport,
            OptionSelectWithPoint,
            OptionHighlightSetEntity,
            OptionEntityGetChildUuid,
            OptionEntityGetNumChildren,
            OptionEntityGetParentId,
            OptionEntityGetAllChildUuids,
            OptionEntityGetSketchPaths,
            OptionLoft,
            OptionClosePath,
            OptionCameraDragMove,
            OptionCameraDragEnd,
            OptionDefaultCameraGetSettings,
            OptionDefaultCameraGetView,
            OptionDefaultCameraSetView,
            OptionDefaultCameraZoom,
            OptionZoomToFit,
            OptionOrientToFace,
            OptionViewIsometric,
            OptionGetNumObjects,
            OptionMakeOffsetPath,
            OptionSetObjectTransform,
            OptionAddHoleFromOffset,
            OptionDefaultCameraFocusOn,
            OptionSelectGet,
            OptionSolid3DGetAllEdgeFaces,
            OptionSolid3DGetAllOppositeEdges,
            OptionSolid3DGetOppositeEdge,
            OptionSolid3DGetNextAdjacentEdge,
            OptionSolid3DGetPrevAdjacentEdge,
            OptionSolid3DGetCommonEdge,
            OptionGetEntityType,
            OptionCurveGetControlPoints,
            OptionProjectEntityToPlane,
            OptionProjectPointsToPlane,
            OptionCurveGetType,
            OptionMouseClick,
            OptionTakeSnapshot,
            OptionPathGetInfo,
            OptionPathSegmentInfo,
            OptionPathGetCurveUuidsForVertices,
            OptionPathGetCurveUuid,
            OptionPathGetVertexUuids,
            OptionPathGetSketchTargetUuid,
            OptionCurveGetEndPoints,
            OptionFaceIsPlanar,
            OptionFaceGetPosition,
            OptionFaceGetCenter,
            OptionFaceGetGradient,
            OptionPlaneIntersectAndProject,
            OptionImportFiles,
            OptionImportedGeometry,
            OptionMass,
            OptionVolume,
            OptionDensity,
            OptionSurfaceArea,
            OptionCenterOfMass,
            OptionGetSketchModePlane,
            OptionEntityGetDistance,
            OptionEntityClone,
            OptionEntityLinearPatternTransform,
            OptionEntityLinearPattern,
            OptionEntityCircularPattern,
            OptionEntityMirror,
            OptionEntityMirrorAcrossEdge,
            OptionEntityMakeHelix,
            OptionEntityMakeHelixFromParams,
            OptionEntityMakeHelixFromEdge,
            OptionSolid3DGetExtrusionFaceInfo,
            OptionExtrusionFaceInfo,
            OptionSetGridReferencePlane,
            OptionBooleanUnion,
            OptionBooleanIntersection,
            OptionBooleanSubtract,
        ],
        Field(discriminator="type"),
    ]
]



from typing import Optional

from pydantic import BaseModel, ConfigDict


class AuthCallback(BaseModel):
    """The authentication callback from the OAuth 2.0 client. This is typically posted to the redirect URL as query params after authenticating."""

    code: Optional[str] = None

    id_token: Optional[str] = None

    state: Optional[str] = None

    user: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.source_range import SourceRange


class SourceRangePrompt(BaseModel):
    """A source range and prompt for a text to CAD iteration."""

    file: Optional[str] = None

    prompt: str

    range: SourceRange

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from .base64data import Base64Data


class DerEncodedKeyPair(BaseModel):
    """The DER encoded key pair."""

    private_key: Base64Data

    public_cert: Base64Data

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict

from ..models.export_file import ExportFile


class Export2d(BaseModel):
    """The response from the `Export2d` endpoint."""

    files: List[ExportFile]

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.text_to_cad_multi_file_iteration import TextToCadMultiFileIteration
from ...models.text_to_cad_multi_file_iteration_body import (
    TextToCadMultiFileIterationBody,
)
from ...types import Response


def _get_kwargs(
    body: TextToCadMultiFileIterationBody,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/ml/text-to-cad/multi-file/iteration".format(
        client.base_url,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "content": body.model_dump_json(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[TextToCadMultiFileIteration, Error]]:
    if response.status_code == 201:
        response_201 = TextToCadMultiFileIteration(**response.json())
        return response_201
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[TextToCadMultiFileIteration, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    body: TextToCadMultiFileIterationBody,
    *,
    client: Client,
) -> Response[Optional[Union[TextToCadMultiFileIteration, Error]]]:
    kwargs = _get_kwargs(
        body=body,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    body: TextToCadMultiFileIterationBody,
    *,
    client: Client,
) -> Optional[Union[TextToCadMultiFileIteration, Error]]:
    """This endpoint can iterate on multi-file models.

    Even if you give specific ranges to edit, the model might change more than just those in order to make the changes you requested without breaking the code.

    You always get the whole code back, even if you only changed a small part of it. This endpoint will always return all the code back, including files that were not changed. If your original source code imported a stl/gltf/step/etc file, the output will not include that file since the model will never change non-kcl files. The endpoint will only return the kcl files that were changed.

    This operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint."""  # noqa: E501

    return sync_detailed(
        body=body,
        client=client,
    ).parsed


async def asyncio_detailed(
    body: TextToCadMultiFileIterationBody,
    *,
    client: Client,
) -> Response[Optional[Union[TextToCadMultiFileIteration, Error]]]:
    kwargs = _get_kwargs(
        body=body,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    body: TextToCadMultiFileIterationBody,
    *,
    client: Client,
) -> Optional[Union[TextToCadMultiFileIteration, Error]]:
    """This endpoint can iterate on multi-file models.

    Even if you give specific ranges to edit, the model might change more than just those in order to make the changes you requested without breaking the code.

    You always get the whole code back, even if you only changed a small part of it. This endpoint will always return all the code back, including files that were not changed. If your original source code imported a stl/gltf/step/etc file, the output will not include that file since the model will never change non-kcl files. The endpoint will only return the kcl files that were changed.

    This operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint."""  # noqa: E501

    return (
        await asyncio_detailed(
            body=body,
            client=client,
        )
    ).parsed



from pydantic import BaseModel, ConfigDict


class CreateShortlinkResponse(BaseModel):
    """Response from creating a shortlink."""

    key: str

    url: str

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.camera_settings import CameraSettings


class DefaultCameraZoom(BaseModel):
    """The response from the `DefaultCameraZoom` command."""

    settings: CameraSettings

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.modeling_app_organization_subscription_tier import (
    ModelingAppOrganizationSubscriptionTier,
)


class ZooProductSubscriptionsOrgRequest(BaseModel):
    """A struct of Zoo product subscriptions an organization can request."""

    modeling_app: ModelingAppOrganizationSubscriptionTier = "team"  # type: ignore

    pay_annually: Optional[bool] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class ClientMetrics(BaseModel):
    """ClientMetrics contains information regarding the state of the peer."""

    rtc_frame_height: Optional[int] = None

    rtc_frame_width: Optional[int] = None

    rtc_frames_decoded: Optional[int] = None

    rtc_frames_dropped: Optional[int] = None

    rtc_frames_per_second: Optional[int] = None

    rtc_frames_received: Optional[int] = None

    rtc_freeze_count: Optional[int] = None

    rtc_jitter_sec: Optional[float] = None

    rtc_keyframes_decoded: Optional[int] = None

    rtc_packets_lost: Optional[int] = None

    rtc_pause_count: Optional[int] = None

    rtc_pli_count: Optional[int] = None

    rtc_stun_rtt_sec: Optional[float] = None

    rtc_total_freezes_duration_sec: Optional[float] = None

    rtc_total_pauses_duration_sec: Optional[float] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict


class Cluster(BaseModel):
    """Cluster information."""

    addr: Optional[str] = None

    auth_timeout: int = 0

    cluster_port: int = 0

    name: str = ""

    tls_timeout: int = 0

    urls: List[str] = []

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.app_client_info import AppClientInfo
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/apps/github/consent".format(
        client.base_url,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[AppClientInfo, Error]]:
    if response.status_code == 200:
        response_200 = AppClientInfo(**response.json())
        return response_200
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[AppClientInfo, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[Optional[Union[AppClientInfo, Error]]]:
    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
) -> Optional[Union[AppClientInfo, Error]]:
    """This is different than OAuth 2.0 authentication for users. This endpoint grants access for Zoo to access user's repos.

    The user doesn't need Zoo OAuth authorization for this endpoint, this is purely for the GitHub permissions to access repos."""  # noqa: E501

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[Optional[Union[AppClientInfo, Error]]]:
    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
) -> Optional[Union[AppClientInfo, Error]]:
    """This is different than OAuth 2.0 authentication for users. This endpoint grants access for Zoo to access user's repos.

    The user doesn't need Zoo OAuth authorization for this endpoint, this is purely for the GitHub permissions to access repos."""  # noqa: E501

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



import datetime
from typing import Dict, Optional

from pydantic import BaseModel, ConfigDict

from ..models.api_call_status import ApiCallStatus
from ..models.file_export_format import FileExportFormat
from ..models.file_import_format import FileImportFormat
from ..models.input_format3d import InputFormat3d
from ..models.output_format3d import OutputFormat3d
from ..models.uuid import Uuid
from .base64data import Base64Data


class FileConversion(BaseModel):
    """A file conversion."""

    completed_at: Optional[datetime.datetime] = None

    created_at: datetime.datetime

    error: Optional[str] = None

    id: Uuid

    output_format: FileExportFormat

    output_format_options: Optional[OutputFormat3d] = None

    outputs: Optional[Dict[str, Base64Data]] = None

    src_format: FileImportFormat

    src_format_options: Optional[InputFormat3d] = None

    started_at: Optional[datetime.datetime] = None

    status: ApiCallStatus

    updated_at: datetime.datetime

    user_id: Uuid

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class Direction(str, Enum):
    """Specifies the sign of a co-ordinate axis."""  # noqa: E501

    """# Increasing numbers. """  # noqa: E501
    POSITIVE = "positive"
    """# Decreasing numbers. """  # noqa: E501
    NEGATIVE = "negative"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GlobalAxis(str, Enum):
    """The global axes."""  # noqa: E501

    """# The X axis """  # noqa: E501
    X = "x"
    """# The Y axis """  # noqa: E501
    Y = "y"
    """# The Z axis """  # noqa: E501
    Z = "z"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitLength(str, Enum):
    """The valid types of length units."""  # noqa: E501

    """# Centimeters <https://en.wikipedia.org/wiki/Centimeter> """  # noqa: E501
    CM = "cm"
    """# Feet <https://en.wikipedia.org/wiki/Foot_(unit)> """  # noqa: E501
    FT = "ft"
    """# Inches <https://en.wikipedia.org/wiki/Inch> """  # noqa: E501
    IN = "in"
    """# Meters <https://en.wikipedia.org/wiki/Meter> """  # noqa: E501
    M = "m"
    """# Millimeters <https://en.wikipedia.org/wiki/Millimeter> """  # noqa: E501
    MM = "mm"
    """# Yards <https://en.wikipedia.org/wiki/Yard> """  # noqa: E501
    YD = "yd"

    def __str__(self) -> str:
        return str(self.value)



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.annotation_line_end_options import AnnotationLineEndOptions
from ..models.annotation_text_options import AnnotationTextOptions
from ..models.color import Color
from ..models.point3d import Point3d


class AnnotationOptions(BaseModel):
    """Options for annotations"""

    color: Optional[Color] = None

    line_ends: Optional[AnnotationLineEndOptions] = None

    line_width: Optional[float] = None

    position: Optional[Point3d] = None

    text: Optional[AnnotationTextOptions] = None

    model_config = ConfigDict(protected_namespaces=())



import json
from typing import Any, Dict, Iterator, Optional

import bson
from websockets.asyncio.client import (
    ClientConnection as ClientConnectionAsync,
    connect as ws_connect_async,
)
from websockets.sync.client import (
    ClientConnection as ClientConnectionSync,
    connect as ws_connect,
)

from ...client import Client
from ...models.post_effect_type import PostEffectType
from ...models.web_socket_request import WebSocketRequest
from ...models.web_socket_response import WebSocketResponse


def _get_kwargs(
    fps: int,
    post_effect: PostEffectType,
    show_grid: bool,
    unlocked_framerate: bool,
    video_res_height: int,
    video_res_width: int,
    webrtc: bool,
    *,
    client: Client,
    api_call_id: Optional[str] = None,
    pool: Optional[str] = None,
    replay: Optional[str] = None,
) -> Dict[str, Any]:
    url = "{}/ws/modeling/commands".format(client.base_url)  # noqa: E501

    if api_call_id is not None:
        if "?" in url:
            url = url + "&api_call_id=" + str(api_call_id)
        else:
            url = url + "?api_call_id=" + str(api_call_id)

    if fps is not None:
        if "?" in url:
            url = url + "&fps=" + str(fps)
        else:
            url = url + "?fps=" + str(fps)

    if pool is not None:
        if "?" in url:
            url = url + "&pool=" + str(pool)
        else:
            url = url + "?pool=" + str(pool)

    if post_effect is not None:
        if "?" in url:
            url = url + "&post_effect=" + str(post_effect)
        else:
            url = url + "?post_effect=" + str(post_effect)

    if replay is not None:
        if "?" in url:
            url = url + "&replay=" + str(replay)
        else:
            url = url + "?replay=" + str(replay)

    if show_grid is not None:
        if "?" in url:
            url = url + "&show_grid=" + str(show_grid).lower()
        else:
            url = url + "?show_grid=" + str(show_grid).lower()

    if unlocked_framerate is not None:
        if "?" in url:
            url = url + "&unlocked_framerate=" + str(unlocked_framerate).lower()
        else:
            url = url + "?unlocked_framerate=" + str(unlocked_framerate).lower()

    if video_res_height is not None:
        if "?" in url:
            url = url + "&video_res_height=" + str(video_res_height)
        else:
            url = url + "?video_res_height=" + str(video_res_height)

    if video_res_width is not None:
        if "?" in url:
            url = url + "&video_res_width=" + str(video_res_width)
        else:
            url = url + "?video_res_width=" + str(video_res_width)

    if webrtc is not None:
        if "?" in url:
            url = url + "&webrtc=" + str(webrtc).lower()
        else:
            url = url + "?webrtc=" + str(webrtc).lower()

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def sync(
    fps: int,
    post_effect: PostEffectType,
    show_grid: bool,
    unlocked_framerate: bool,
    video_res_height: int,
    video_res_width: int,
    webrtc: bool,
    *,
    client: Client,
    api_call_id: Optional[str] = None,
    pool: Optional[str] = None,
    replay: Optional[str] = None,
) -> ClientConnectionSync:
    """Pass those commands to the engine via websocket, and pass responses back to the client. Basically, this is a websocket proxy between the frontend/client and the engine."""  # noqa: E501

    kwargs = _get_kwargs(
        api_call_id=api_call_id,
        fps=fps,
        pool=pool,
        post_effect=post_effect,
        replay=replay,
        show_grid=show_grid,
        unlocked_framerate=unlocked_framerate,
        video_res_height=video_res_height,
        video_res_width=video_res_width,
        webrtc=webrtc,
        client=client,
    )

    return ws_connect(
        kwargs["url"].replace("http", "ws"),
        additional_headers=kwargs["headers"],
        close_timeout=120,
        max_size=None,
    )  # type: ignore


async def asyncio(
    fps: int,
    post_effect: PostEffectType,
    show_grid: bool,
    unlocked_framerate: bool,
    video_res_height: int,
    video_res_width: int,
    webrtc: bool,
    *,
    client: Client,
    api_call_id: Optional[str] = None,
    pool: Optional[str] = None,
    replay: Optional[str] = None,
) -> ClientConnectionAsync:
    """Pass those commands to the engine via websocket, and pass responses back to the client. Basically, this is a websocket proxy between the frontend/client and the engine."""  # noqa: E501

    kwargs = _get_kwargs(
        api_call_id=api_call_id,
        fps=fps,
        pool=pool,
        post_effect=post_effect,
        replay=replay,
        show_grid=show_grid,
        unlocked_framerate=unlocked_framerate,
        video_res_height=video_res_height,
        video_res_width=video_res_width,
        webrtc=webrtc,
        client=client,
    )

    return await ws_connect_async(
        kwargs["url"].replace("http", "ws"),
        extra_headers=kwargs["headers"],
        close_timeout=120,
        max_size=None,
    )


class WebSocket:
    """A websocket connection to the API endpoint."""

    ws: ClientConnectionSync

    def __init__(
        self,
        fps: int,
        post_effect: PostEffectType,
        show_grid: bool,
        unlocked_framerate: bool,
        video_res_height: int,
        video_res_width: int,
        webrtc: bool,
        client: Client,
        api_call_id: Optional[str] = None,
        pool: Optional[str] = None,
        replay: Optional[str] = None,
    ):
        self.ws = sync(
            fps,
            post_effect,
            show_grid,
            unlocked_framerate,
            video_res_height,
            video_res_width,
            webrtc,
            client=client,
            api_call_id=api_call_id,
            pool=pool,
            replay=replay,
        )

    def __enter__(
        self,
    ):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def __iter__(self) -> Iterator[WebSocketResponse]:
        """
        Iterate on incoming messages.

        The iterator calls :meth:`recv` and yields messages in an infinite loop.

        It exits when the connection is closed normally. It raises a
        :exc:`~websockets.exceptions.ConnectionClosedError` exception after a
        protocol error or a network failure.

        """
        for message in self.ws:
            yield WebSocketResponse(**json.loads(message))

    def send(self, data: WebSocketRequest):
        """Send data to the websocket."""
        self.ws.send(json.dumps(data.model_dump()))

    def send_binary(self, data: WebSocketRequest):
        """Send data as bson to the websocket."""
        self.ws.send(bson.encode(data.model_dump()))  # type: ignore

    def recv(self) -> WebSocketResponse:
        """Receive data from the websocket."""
        message = self.ws.recv(timeout=60)
        return WebSocketResponse(**json.loads(message))

    def close(self):
        """Close the websocket."""
        self.ws.close()



from typing import List, Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.angle import Angle
from ..models.annotation_options import AnnotationOptions
from ..models.annotation_type import AnnotationType
from ..models.camera_drag_interaction_type import CameraDragInteractionType
from ..models.camera_movement import CameraMovement
from ..models.camera_view_state import CameraViewState
from ..models.color import Color
from ..models.component_transform import ComponentTransform
from ..models.cut_type import CutType
from ..models.distance_type import DistanceType
from ..models.entity_type import EntityType
from ..models.extruded_face_info import ExtrudedFaceInfo
from ..models.image_format import ImageFormat
from ..models.import_file import ImportFile
from ..models.input_format3d import InputFormat3d
from ..models.length_unit import LengthUnit
from ..models.modeling_cmd_id import ModelingCmdId
from ..models.opposite_for_angle import OppositeForAngle
from ..models.opposite_for_length_unit import OppositeForLengthUnit
from ..models.output_format2d import OutputFormat2d
from ..models.output_format3d import OutputFormat3d
from ..models.path_component_constraint_bound import PathComponentConstraintBound
from ..models.path_component_constraint_type import PathComponentConstraintType
from ..models.path_segment import PathSegment
from ..models.perspective_camera_parameters import PerspectiveCameraParameters
from ..models.point2d import Point2d
from ..models.point3d import Point3d
from ..models.scene_selection_type import SceneSelectionType
from ..models.scene_tool_type import SceneToolType
from ..models.transform import Transform
from ..models.unit_area import UnitArea
from ..models.unit_density import UnitDensity
from ..models.unit_length import UnitLength
from ..models.unit_mass import UnitMass
from ..models.unit_volume import UnitVolume


class OptionEngineUtilEvaluatePath(BaseModel):
    """Evaluates the position of a path in one shot (engine utility for kcl executor)"""

    path_json: str

    t: float

    type: Literal["engine_util_evaluate_path"] = "engine_util_evaluate_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionStartPath(BaseModel):
    """Start a new path."""

    type: Literal["start_path"] = "start_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionMovePathPen(BaseModel):
    """Move the path's \"pen\". If you're in sketch mode, these coordinates are in the local coordinate system, not the world's coordinate system. For example, say you're sketching on the plane {x: (1,0,0), y: (0,1,0), origin: (0, 0, 50)}. In other words, the plane 50 units above the default XY plane. Then, moving the pen to (1, 1, 0) with this command uses local coordinates. So, it would move the pen to (1, 1, 50) in global coordinates."""

    path: ModelingCmdId

    to: Point3d

    type: Literal["move_path_pen"] = "move_path_pen"

    model_config = ConfigDict(protected_namespaces=())


class OptionExtendPath(BaseModel):
    """Extend a path by adding a new segment which starts at the path's \"pen\". If no \"pen\" location has been set before (via `MovePen`), then the pen is at the origin."""

    path: ModelingCmdId

    segment: PathSegment

    type: Literal["extend_path"] = "extend_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionExtrude(BaseModel):
    """Command for extruding a solid 2d."""

    distance: LengthUnit

    faces: Optional[ExtrudedFaceInfo] = None

    opposite: OppositeForLengthUnit = "None"  # type: ignore

    target: ModelingCmdId

    type: Literal["extrude"] = "extrude"

    model_config = ConfigDict(protected_namespaces=())


class OptionSweep(BaseModel):
    """Extrude the object along a path."""

    sectional: bool

    target: ModelingCmdId

    tolerance: LengthUnit

    trajectory: ModelingCmdId

    type: Literal["sweep"] = "sweep"

    model_config = ConfigDict(protected_namespaces=())


class OptionRevolve(BaseModel):
    """Command for revolving a solid 2d."""

    angle: Angle

    axis: Point3d

    axis_is_2d: bool

    opposite: OppositeForAngle = "None"  # type: ignore

    origin: Point3d

    target: ModelingCmdId

    tolerance: LengthUnit

    type: Literal["revolve"] = "revolve"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DShellFace(BaseModel):
    """Command for shelling a solid3d face"""

    face_ids: List[str]

    hollow: bool = False

    object_id: str

    shell_thickness: LengthUnit

    type: Literal["solid3d_shell_face"] = "solid3d_shell_face"

    model_config = ConfigDict(protected_namespaces=())


class OptionRevolveAboutEdge(BaseModel):
    """Command for revolving a solid 2d about a brep edge"""

    angle: Angle

    edge_id: str

    opposite: OppositeForAngle = "None"  # type: ignore

    target: ModelingCmdId

    tolerance: LengthUnit

    type: Literal["revolve_about_edge"] = "revolve_about_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionLoft(BaseModel):
    """Command for lofting sections to create a solid"""

    base_curve_index: Optional[int] = None

    bez_approximate_rational: bool

    section_ids: List[str]

    tolerance: LengthUnit

    type: Literal["loft"] = "loft"

    v_degree: int

    model_config = ConfigDict(protected_namespaces=())


class OptionClosePath(BaseModel):
    """Closes a path, converting it to a 2D solid."""

    path_id: str

    type: Literal["close_path"] = "close_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionCameraDragStart(BaseModel):
    """Camera drag started."""

    interaction: CameraDragInteractionType

    type: Literal["camera_drag_start"] = "camera_drag_start"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionCameraDragMove(BaseModel):
    """Camera drag continued."""

    interaction: CameraDragInteractionType

    sequence: Optional[int] = None

    type: Literal["camera_drag_move"] = "camera_drag_move"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionCameraDragEnd(BaseModel):
    """Camera drag ended"""

    interaction: CameraDragInteractionType

    type: Literal["camera_drag_end"] = "camera_drag_end"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraGetSettings(BaseModel):
    """Gets the default camera's camera settings"""

    type: Literal["default_camera_get_settings"] = "default_camera_get_settings"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraGetView(BaseModel):
    """Gets the default camera's view state"""

    type: Literal["default_camera_get_view"] = "default_camera_get_view"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraSetView(BaseModel):
    """Sets the default camera's view state"""

    type: Literal["default_camera_set_view"] = "default_camera_set_view"

    view: CameraViewState

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraLookAt(BaseModel):
    """Change what the default camera is looking at."""

    center: Point3d

    sequence: Optional[int] = None

    type: Literal["default_camera_look_at"] = "default_camera_look_at"

    up: Point3d

    vantage: Point3d

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraPerspectiveSettings(BaseModel):
    """Change what the default camera is looking at."""

    center: Point3d

    fov_y: Optional[float] = None

    sequence: Optional[int] = None

    type: Literal["default_camera_perspective_settings"] = (
        "default_camera_perspective_settings"
    )

    up: Point3d

    vantage: Point3d

    z_far: Optional[float] = None

    z_near: Optional[float] = None

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraZoom(BaseModel):
    """Adjust zoom of the default camera."""

    magnitude: float

    type: Literal["default_camera_zoom"] = "default_camera_zoom"

    model_config = ConfigDict(protected_namespaces=())


class OptionExport2D(BaseModel):
    """Export a sketch to a file."""

    entity_ids: List[str]

    format: OutputFormat2d

    type: Literal["export2d"] = "export2d"

    model_config = ConfigDict(protected_namespaces=())


class OptionExport3D(BaseModel):
    """Export the scene to a file."""

    entity_ids: List[str]

    format: OutputFormat3d

    type: Literal["export3d"] = "export3d"

    model_config = ConfigDict(protected_namespaces=())


class OptionExport(BaseModel):
    """Export the scene to a file."""

    entity_ids: List[str]

    format: OutputFormat3d

    type: Literal["export"] = "export"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetParentId(BaseModel):
    """What is this entity's parent?"""

    entity_id: str

    type: Literal["entity_get_parent_id"] = "entity_get_parent_id"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetNumChildren(BaseModel):
    """How many children does the entity have?"""

    entity_id: str

    type: Literal["entity_get_num_children"] = "entity_get_num_children"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetChildUuid(BaseModel):
    """What is the UUID of this entity's n-th child?"""

    child_index: int

    entity_id: str

    type: Literal["entity_get_child_uuid"] = "entity_get_child_uuid"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetAllChildUuids(BaseModel):
    """What are all UUIDs of this entity's children?"""

    entity_id: str

    type: Literal["entity_get_all_child_uuids"] = "entity_get_all_child_uuids"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetSketchPaths(BaseModel):
    """What are all UUIDs of all the paths sketched on top of this entity?"""

    entity_id: str

    type: Literal["entity_get_sketch_paths"] = "entity_get_sketch_paths"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityGetDistance(BaseModel):
    """What is the distance between these two entities?"""

    distance_type: DistanceType

    entity_id1: str

    entity_id2: str

    type: Literal["entity_get_distance"] = "entity_get_distance"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityClone(BaseModel):
    """Create a pattern using this entity by specifying the transform for each desired repetition. Transformations are performed in the following order (first applied to last applied): scale, rotate, translate."""

    entity_id: str

    type: Literal["entity_clone"] = "entity_clone"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityLinearPatternTransform(BaseModel):
    """Create a pattern using this entity by specifying the transform for each desired repetition. Transformations are performed in the following order (first applied to last applied): scale, rotate, translate."""

    entity_id: str

    transform: List[Transform] = []

    transforms: List[List[Transform]] = []

    type: Literal["entity_linear_pattern_transform"] = "entity_linear_pattern_transform"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityLinearPattern(BaseModel):
    """Create a linear pattern using this entity."""

    axis: Point3d

    entity_id: str

    num_repetitions: int

    spacing: LengthUnit

    type: Literal["entity_linear_pattern"] = "entity_linear_pattern"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityCircularPattern(BaseModel):
    """Create a circular pattern using this entity."""

    arc_degrees: float

    axis: Point3d

    center: Point3d

    entity_id: str

    num_repetitions: int

    rotate_duplicates: bool

    type: Literal["entity_circular_pattern"] = "entity_circular_pattern"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMakeHelix(BaseModel):
    """Create a helix using the input cylinder and other specified parameters."""

    cylinder_id: str

    is_clockwise: bool

    length: LengthUnit

    revolutions: float

    start_angle: Angle = {"unit": "degrees", "value": 0.0}  # type: ignore

    type: Literal["entity_make_helix"] = "entity_make_helix"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMakeHelixFromParams(BaseModel):
    """Create a helix using the specified parameters."""

    axis: Point3d

    center: Point3d

    is_clockwise: bool

    length: LengthUnit

    radius: LengthUnit

    revolutions: float

    start_angle: Angle = {"unit": "degrees", "value": 0.0}  # type: ignore

    type: Literal["entity_make_helix_from_params"] = "entity_make_helix_from_params"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMakeHelixFromEdge(BaseModel):
    """Create a helix using the specified parameters."""

    edge_id: str

    is_clockwise: bool

    length: Optional[LengthUnit] = None

    radius: LengthUnit

    revolutions: float

    start_angle: Angle = {"unit": "degrees", "value": 0.0}  # type: ignore

    type: Literal["entity_make_helix_from_edge"] = "entity_make_helix_from_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMirror(BaseModel):
    """Mirror the input entities over the specified axis. (Currently only supports sketches)"""

    axis: Point3d

    ids: List[str]

    point: Point3d

    type: Literal["entity_mirror"] = "entity_mirror"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityMirrorAcrossEdge(BaseModel):
    """Mirror the input entities over the specified edge. (Currently only supports sketches)"""

    edge_id: str

    ids: List[str]

    type: Literal["entity_mirror_across_edge"] = "entity_mirror_across_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectWithPoint(BaseModel):
    """Modifies the selection by simulating a \"mouse click\" at the given x,y window coordinate Returns ID of whatever was selected."""

    selected_at_window: Point2d

    selection_type: SceneSelectionType

    type: Literal["select_with_point"] = "select_with_point"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectAdd(BaseModel):
    """Adds one or more entities (by UUID) to the selection."""

    entities: List[str]

    type: Literal["select_add"] = "select_add"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectRemove(BaseModel):
    """Removes one or more entities (by UUID) from the selection."""

    entities: List[str]

    type: Literal["select_remove"] = "select_remove"

    model_config = ConfigDict(protected_namespaces=())


class OptionSceneClearAll(BaseModel):
    """Removes all of the Objects in the scene"""

    type: Literal["scene_clear_all"] = "scene_clear_all"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectReplace(BaseModel):
    """Replaces current selection with these entities (by UUID)."""

    entities: List[str]

    type: Literal["select_replace"] = "select_replace"

    model_config = ConfigDict(protected_namespaces=())


class OptionHighlightSetEntity(BaseModel):
    """Changes the current highlighted entity to whichever one is at the given window coordinate. If there's no entity at this location, clears the highlight."""

    selected_at_window: Point2d

    sequence: Optional[int] = None

    type: Literal["highlight_set_entity"] = "highlight_set_entity"

    model_config = ConfigDict(protected_namespaces=())


class OptionHighlightSetEntities(BaseModel):
    """Changes the current highlighted entity to these entities."""

    entities: List[str]

    type: Literal["highlight_set_entities"] = "highlight_set_entities"

    model_config = ConfigDict(protected_namespaces=())


class OptionNewAnnotation(BaseModel):
    """Create a new annotation"""

    annotation_type: AnnotationType

    clobber: bool

    options: AnnotationOptions

    type: Literal["new_annotation"] = "new_annotation"

    model_config = ConfigDict(protected_namespaces=())


class OptionUpdateAnnotation(BaseModel):
    """Update an annotation"""

    annotation_id: str

    options: AnnotationOptions

    type: Literal["update_annotation"] = "update_annotation"

    model_config = ConfigDict(protected_namespaces=())


class OptionEdgeLinesVisible(BaseModel):
    """Changes visibility of scene-wide edge lines on brep solids"""

    hidden: bool

    type: Literal["edge_lines_visible"] = "edge_lines_visible"

    model_config = ConfigDict(protected_namespaces=())


class OptionObjectVisible(BaseModel):
    """Hide or show an object"""

    hidden: bool

    object_id: str

    type: Literal["object_visible"] = "object_visible"

    model_config = ConfigDict(protected_namespaces=())


class OptionObjectBringToFront(BaseModel):
    """Bring an object to the front of the scene"""

    object_id: str

    type: Literal["object_bring_to_front"] = "object_bring_to_front"

    model_config = ConfigDict(protected_namespaces=())


class OptionObjectSetMaterialParamsPbr(BaseModel):
    """Set the material properties of an object"""

    ambient_occlusion: float

    color: Color

    metalness: float

    object_id: str

    roughness: float

    type: Literal["object_set_material_params_pbr"] = "object_set_material_params_pbr"

    model_config = ConfigDict(protected_namespaces=())


class OptionGetEntityType(BaseModel):
    """What type of entity is this?"""

    entity_id: str

    type: Literal["get_entity_type"] = "get_entity_type"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetAllEdgeFaces(BaseModel):
    """Gets all faces which use the given edge."""

    edge_id: str

    object_id: str

    type: Literal["solid3d_get_all_edge_faces"] = "solid3d_get_all_edge_faces"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid2DAddHole(BaseModel):
    """Add a hole to a Solid2d object before extruding it."""

    hole_id: str

    object_id: str

    type: Literal["solid2d_add_hole"] = "solid2d_add_hole"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetAllOppositeEdges(BaseModel):
    """Gets all edges which are opposite the given edge, across all possible faces."""

    along_vector: Optional[Point3d] = None

    edge_id: str

    object_id: str

    type: Literal["solid3d_get_all_opposite_edges"] = "solid3d_get_all_opposite_edges"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetOppositeEdge(BaseModel):
    """Gets the edge opposite the given edge, along the given face."""

    edge_id: str

    face_id: str

    object_id: str

    type: Literal["solid3d_get_opposite_edge"] = "solid3d_get_opposite_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetNextAdjacentEdge(BaseModel):
    """Gets the next adjacent edge for the given edge, along the given face."""

    edge_id: str

    face_id: str

    object_id: str

    type: Literal["solid3d_get_next_adjacent_edge"] = "solid3d_get_next_adjacent_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetPrevAdjacentEdge(BaseModel):
    """Gets the previous adjacent edge for the given edge, along the given face."""

    edge_id: str

    face_id: str

    object_id: str

    type: Literal["solid3d_get_prev_adjacent_edge"] = "solid3d_get_prev_adjacent_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetCommonEdge(BaseModel):
    """Gets the shared edge between these two faces if it exists"""

    face_ids: List[str]

    object_id: str

    type: Literal["solid3d_get_common_edge"] = "solid3d_get_common_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DFilletEdge(BaseModel):
    """Fillets the given edge with the specified radius."""

    cut_type: CutType = "fillet"  # type: ignore

    edge_id: str

    object_id: str

    radius: LengthUnit

    tolerance: LengthUnit

    type: Literal["solid3d_fillet_edge"] = "solid3d_fillet_edge"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceIsPlanar(BaseModel):
    """Determines whether a brep face is planar and returns its surface-local planar axes if so"""

    object_id: str

    type: Literal["face_is_planar"] = "face_is_planar"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceGetPosition(BaseModel):
    """Determines a position on a brep face evaluated by parameters u,v"""

    object_id: str

    type: Literal["face_get_position"] = "face_get_position"

    uv: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceGetCenter(BaseModel):
    """Obtains the surface \"center of mass\" """

    object_id: str

    type: Literal["face_get_center"] = "face_get_center"

    model_config = ConfigDict(protected_namespaces=())


class OptionFaceGetGradient(BaseModel):
    """Determines the gradient (dFdu, dFdv) + normal vector on a brep face evaluated by parameters u,v"""

    object_id: str

    type: Literal["face_get_gradient"] = "face_get_gradient"

    uv: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionSendObject(BaseModel):
    """Send object to front or back."""

    front: bool

    object_id: str

    type: Literal["send_object"] = "send_object"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntitySetOpacity(BaseModel):
    """Set opacity of the entity."""

    entity_id: str

    opacity: float

    type: Literal["entity_set_opacity"] = "entity_set_opacity"

    model_config = ConfigDict(protected_namespaces=())


class OptionEntityFade(BaseModel):
    """Fade entity in or out."""

    duration_seconds: float = 0.4000000059604645

    entity_id: str

    fade_in: bool

    type: Literal["entity_fade"] = "entity_fade"

    model_config = ConfigDict(protected_namespaces=())


class OptionMakePlane(BaseModel):
    """Make a new plane"""

    clobber: bool

    hide: Optional[bool] = None

    origin: Point3d

    size: LengthUnit

    type: Literal["make_plane"] = "make_plane"

    x_axis: Point3d

    y_axis: Point3d

    model_config = ConfigDict(protected_namespaces=())


class OptionPlaneSetColor(BaseModel):
    """Set the color of a plane."""

    color: Color

    plane_id: str

    type: Literal["plane_set_color"] = "plane_set_color"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetTool(BaseModel):
    """Set the current tool."""

    tool: SceneToolType

    type: Literal["set_tool"] = "set_tool"

    model_config = ConfigDict(protected_namespaces=())


class OptionMouseMove(BaseModel):
    """Send a mouse move event"""

    sequence: Optional[int] = None

    type: Literal["mouse_move"] = "mouse_move"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionMouseClick(BaseModel):
    """Send a mouse click event Updates modified/selected entities."""

    type: Literal["mouse_click"] = "mouse_click"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionSketchModeDisable(BaseModel):
    """Disable sketch mode. If you are sketching on a face, be sure to not disable sketch mode until you have extruded. Otherwise, your object will not be fused with the face."""

    type: Literal["sketch_mode_disable"] = "sketch_mode_disable"

    model_config = ConfigDict(protected_namespaces=())


class OptionGetSketchModePlane(BaseModel):
    """Get the plane for sketch mode."""

    type: Literal["get_sketch_mode_plane"] = "get_sketch_mode_plane"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveSetConstraint(BaseModel):
    """Get the plane for sketch mode."""

    constraint_bound: PathComponentConstraintBound

    constraint_type: PathComponentConstraintType

    object_id: str

    type: Literal["curve_set_constraint"] = "curve_set_constraint"

    model_config = ConfigDict(protected_namespaces=())


class OptionEnableSketchMode(BaseModel):
    """Sketch on some entity (e.g. a plane, a face)."""

    adjust_camera: bool

    animated: bool

    entity_id: str

    ortho: bool

    planar_normal: Optional[Point3d] = None

    type: Literal["enable_sketch_mode"] = "enable_sketch_mode"

    model_config = ConfigDict(protected_namespaces=())


class OptionEnableDryRun(BaseModel):
    """Sets whether or not changes to the scene or its objects will be done as a \"dry run\" In a dry run, successful commands won't actually change the model. This is useful for catching errors before actually making the change."""

    type: Literal["enable_dry_run"] = "enable_dry_run"

    model_config = ConfigDict(protected_namespaces=())


class OptionDisableDryRun(BaseModel):
    """Sets whether or not changes to the scene or its objects will be done as a \"dry run\" In a dry run, successful commands won't actually change the model. This is useful for catching errors before actually making the change."""

    type: Literal["disable_dry_run"] = "disable_dry_run"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetBackgroundColor(BaseModel):
    """Set the background color of the scene."""

    color: Color

    type: Literal["set_background_color"] = "set_background_color"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetCurrentToolProperties(BaseModel):
    """Set the properties of the tool lines for the scene."""

    color: Optional[Color] = None

    type: Literal["set_current_tool_properties"] = "set_current_tool_properties"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetDefaultSystemProperties(BaseModel):
    """Set the default system properties used when a specific property isn't set."""

    color: Optional[Color] = None

    type: Literal["set_default_system_properties"] = "set_default_system_properties"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveGetType(BaseModel):
    """Get type of the given curve."""

    curve_id: str

    type: Literal["curve_get_type"] = "curve_get_type"

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveGetControlPoints(BaseModel):
    """Get control points of the given curve."""

    curve_id: str

    type: Literal["curve_get_control_points"] = "curve_get_control_points"

    model_config = ConfigDict(protected_namespaces=())


class OptionProjectEntityToPlane(BaseModel):
    """Project an entity on to a plane."""

    entity_id: str

    plane_id: str

    type: Literal["project_entity_to_plane"] = "project_entity_to_plane"

    use_plane_coords: bool

    model_config = ConfigDict(protected_namespaces=())


class OptionProjectPointsToPlane(BaseModel):
    """Project a list of points on to a plane."""

    plane_id: str

    points: List[Point3d]

    type: Literal["project_points_to_plane"] = "project_points_to_plane"

    use_plane_coords: bool

    model_config = ConfigDict(protected_namespaces=())


class OptionTakeSnapshot(BaseModel):
    """Take a snapshot of the current view."""

    format: ImageFormat

    type: Literal["take_snapshot"] = "take_snapshot"

    model_config = ConfigDict(protected_namespaces=())


class OptionMakeAxesGizmo(BaseModel):
    """Add a gizmo showing the axes."""

    clobber: bool

    gizmo_mode: bool

    type: Literal["make_axes_gizmo"] = "make_axes_gizmo"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetInfo(BaseModel):
    """Query the given path."""

    path_id: str

    type: Literal["path_get_info"] = "path_get_info"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetCurveUuidsForVertices(BaseModel):
    """Obtain curve ids for vertex ids"""

    path_id: str

    type: Literal["path_get_curve_uuids_for_vertices"] = (
        "path_get_curve_uuids_for_vertices"
    )

    vertex_ids: List[str]

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetCurveUuid(BaseModel):
    """Obtain curve id by index"""

    index: int

    path_id: str

    type: Literal["path_get_curve_uuid"] = "path_get_curve_uuid"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetVertexUuids(BaseModel):
    """Obtain vertex ids for a path"""

    path_id: str

    type: Literal["path_get_vertex_uuids"] = "path_get_vertex_uuids"

    model_config = ConfigDict(protected_namespaces=())


class OptionPathGetSketchTargetUuid(BaseModel):
    """Obtain the sketch target id (if the path was drawn in sketchmode) for a path"""

    path_id: str

    type: Literal["path_get_sketch_target_uuid"] = "path_get_sketch_target_uuid"

    model_config = ConfigDict(protected_namespaces=())


class OptionHandleMouseDragStart(BaseModel):
    """Start dragging the mouse."""

    type: Literal["handle_mouse_drag_start"] = "handle_mouse_drag_start"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionHandleMouseDragMove(BaseModel):
    """Continue dragging the mouse."""

    sequence: Optional[int] = None

    type: Literal["handle_mouse_drag_move"] = "handle_mouse_drag_move"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionHandleMouseDragEnd(BaseModel):
    """Stop dragging the mouse."""

    type: Literal["handle_mouse_drag_end"] = "handle_mouse_drag_end"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionRemoveSceneObjects(BaseModel):
    """Remove scene objects."""

    object_ids: List[str]

    type: Literal["remove_scene_objects"] = "remove_scene_objects"

    model_config = ConfigDict(protected_namespaces=())


class OptionPlaneIntersectAndProject(BaseModel):
    """Utility method. Performs both a ray cast and projection to plane-local coordinates. Returns the plane coordinates for the given window coordinates."""

    plane_id: str

    type: Literal["plane_intersect_and_project"] = "plane_intersect_and_project"

    window: Point2d

    model_config = ConfigDict(protected_namespaces=())


class OptionCurveGetEndPoints(BaseModel):
    """Find the start and end of a curve."""

    curve_id: str

    type: Literal["curve_get_end_points"] = "curve_get_end_points"

    model_config = ConfigDict(protected_namespaces=())


class OptionReconfigureStream(BaseModel):
    """Reconfigure the stream."""

    bitrate: Optional[int] = None

    fps: int

    height: int

    type: Literal["reconfigure_stream"] = "reconfigure_stream"

    width: int

    model_config = ConfigDict(protected_namespaces=())


class OptionImportFiles(BaseModel):
    """Import files to the current model."""

    files: List[ImportFile]

    format: InputFormat3d

    type: Literal["import_files"] = "import_files"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetSceneUnits(BaseModel):
    """Set the units of the scene. For all following commands, the units will be interpreted as the given units. Any previously executed commands will not be affected or have their units changed. They will remain in the units they were originally executed in."""

    type: Literal["set_scene_units"] = "set_scene_units"

    unit: UnitLength

    model_config = ConfigDict(protected_namespaces=())


class OptionMass(BaseModel):
    """Get the mass of entities in the scene or the default scene."""

    entity_ids: List[str]

    material_density: float

    material_density_unit: UnitDensity

    output_unit: UnitMass

    type: Literal["mass"] = "mass"

    model_config = ConfigDict(protected_namespaces=())


class OptionDensity(BaseModel):
    """Get the density of entities in the scene or the default scene."""

    entity_ids: List[str]

    material_mass: float

    material_mass_unit: UnitMass

    output_unit: UnitDensity

    type: Literal["density"] = "density"

    model_config = ConfigDict(protected_namespaces=())


class OptionVolume(BaseModel):
    """Get the volume of entities in the scene or the default scene."""

    entity_ids: List[str]

    output_unit: UnitVolume

    type: Literal["volume"] = "volume"

    model_config = ConfigDict(protected_namespaces=())


class OptionCenterOfMass(BaseModel):
    """Get the center of mass of entities in the scene or the default scene."""

    entity_ids: List[str]

    output_unit: UnitLength

    type: Literal["center_of_mass"] = "center_of_mass"

    model_config = ConfigDict(protected_namespaces=())


class OptionSurfaceArea(BaseModel):
    """Get the surface area of entities in the scene or the default scene."""

    entity_ids: List[str]

    output_unit: UnitArea

    type: Literal["surface_area"] = "surface_area"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraFocusOn(BaseModel):
    """Focus the default camera upon an object in the scene."""

    type: Literal["default_camera_focus_on"] = "default_camera_focus_on"

    uuid: str

    model_config = ConfigDict(protected_namespaces=())


class OptionSetSelectionType(BaseModel):
    """When you select some entity with the current tool, what should happen to the entity?"""

    selection_type: SceneSelectionType

    type: Literal["set_selection_type"] = "set_selection_type"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetSelectionFilter(BaseModel):
    """What kind of entities can be selected?"""

    filter: List[EntityType]

    type: Literal["set_selection_filter"] = "set_selection_filter"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraSetOrthographic(BaseModel):
    """Use orthographic projection."""

    type: Literal["default_camera_set_orthographic"] = "default_camera_set_orthographic"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraSetPerspective(BaseModel):
    """Use perspective projection."""

    parameters: Optional[PerspectiveCameraParameters] = None

    type: Literal["default_camera_set_perspective"] = "default_camera_set_perspective"

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraCenterToSelection(BaseModel):
    """Updates the camera to center to the center of the current selection (or the origin if nothing is selected)"""

    camera_movement: CameraMovement = "vantage"  # type: ignore

    type: Literal["default_camera_center_to_selection"] = (
        "default_camera_center_to_selection"
    )

    model_config = ConfigDict(protected_namespaces=())


class OptionDefaultCameraCenterToScene(BaseModel):
    """Updates the camera to center to the center of the current scene's bounds"""

    camera_movement: CameraMovement = "vantage"  # type: ignore

    type: Literal["default_camera_center_to_scene"] = "default_camera_center_to_scene"

    model_config = ConfigDict(protected_namespaces=())


class OptionZoomToFit(BaseModel):
    """Fit the view to the specified object(s)."""

    animated: bool = False

    object_ids: List[str] = []

    padding: float = 0.0

    type: Literal["zoom_to_fit"] = "zoom_to_fit"

    model_config = ConfigDict(protected_namespaces=())


class OptionOrientToFace(BaseModel):
    """Looks along the normal of the specified face (if it is planar!), and fits the view to it."""

    animated: bool = False

    face_id: str

    padding: float = 0.0

    type: Literal["orient_to_face"] = "orient_to_face"

    model_config = ConfigDict(protected_namespaces=())


class OptionViewIsometric(BaseModel):
    """Fit the view to the scene with an isometric view."""

    padding: float = 0.0

    type: Literal["view_isometric"] = "view_isometric"

    model_config = ConfigDict(protected_namespaces=())


class OptionSolid3DGetExtrusionFaceInfo(BaseModel):
    """Get a concise description of all of an extrusion's faces."""

    edge_id: str

    object_id: str

    type: Literal["solid3d_get_extrusion_face_info"] = "solid3d_get_extrusion_face_info"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectClear(BaseModel):
    """Clear the selection"""

    type: Literal["select_clear"] = "select_clear"

    model_config = ConfigDict(protected_namespaces=())


class OptionSelectGet(BaseModel):
    """Find all IDs of selected entities"""

    type: Literal["select_get"] = "select_get"

    model_config = ConfigDict(protected_namespaces=())


class OptionGetNumObjects(BaseModel):
    """Get the number of objects in the scene"""

    type: Literal["get_num_objects"] = "get_num_objects"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetObjectTransform(BaseModel):
    """Set the transform of an object."""

    object_id: str

    transforms: List[ComponentTransform]

    type: Literal["set_object_transform"] = "set_object_transform"

    model_config = ConfigDict(protected_namespaces=())


class OptionBooleanUnion(BaseModel):
    """Create a new solid from combining other smaller solids. In other words, every part of the input solids will be included in the output solid."""

    solid_ids: List[str]

    tolerance: LengthUnit

    type: Literal["boolean_union"] = "boolean_union"

    model_config = ConfigDict(protected_namespaces=())


class OptionBooleanIntersection(BaseModel):
    """Create a new solid from intersecting several other solids. In other words, the part of the input solids where they all overlap will be the output solid."""

    solid_ids: List[str]

    tolerance: LengthUnit

    type: Literal["boolean_intersection"] = "boolean_intersection"

    model_config = ConfigDict(protected_namespaces=())


class OptionBooleanSubtract(BaseModel):
    """Create a new solid from subtracting several other solids. The 'target' is what will be cut from. The 'tool' is what will be cut out from 'target'."""

    target_ids: List[str]

    tolerance: LengthUnit

    tool_ids: List[str]

    type: Literal["boolean_subtract"] = "boolean_subtract"

    model_config = ConfigDict(protected_namespaces=())


class OptionMakeOffsetPath(BaseModel):
    """Make a new path by offsetting an object by a given distance. The new path's ID will be the ID of this command."""

    face_id: Optional[str] = None

    object_id: str

    offset: LengthUnit

    type: Literal["make_offset_path"] = "make_offset_path"

    model_config = ConfigDict(protected_namespaces=())


class OptionAddHoleFromOffset(BaseModel):
    """Add a hole to a closed path by offsetting it a uniform distance inward."""

    object_id: str

    offset: LengthUnit

    type: Literal["add_hole_from_offset"] = "add_hole_from_offset"

    model_config = ConfigDict(protected_namespaces=())


class OptionSetGridReferencePlane(BaseModel):
    """Align the grid with a plane or a planar face."""

    grid_id: str

    reference_id: str

    type: Literal["set_grid_reference_plane"] = "set_grid_reference_plane"

    model_config = ConfigDict(protected_namespaces=())


ModelingCmd = RootModel[
    Annotated[
        Union[
            OptionEngineUtilEvaluatePath,
            OptionStartPath,
            OptionMovePathPen,
            OptionExtendPath,
            OptionExtrude,
            OptionSweep,
            OptionRevolve,
            OptionSolid3DShellFace,
            OptionRevolveAboutEdge,
            OptionLoft,
            OptionClosePath,
            OptionCameraDragStart,
            OptionCameraDragMove,
            OptionCameraDragEnd,
            OptionDefaultCameraGetSettings,
            OptionDefaultCameraGetView,
            OptionDefaultCameraSetView,
            OptionDefaultCameraLookAt,
            OptionDefaultCameraPerspectiveSettings,
            OptionDefaultCameraZoom,
            OptionExport2D,
            OptionExport3D,
            OptionExport,
            OptionEntityGetParentId,
            OptionEntityGetNumChildren,
            OptionEntityGetChildUuid,
            OptionEntityGetAllChildUuids,
            OptionEntityGetSketchPaths,
            OptionEntityGetDistance,
            OptionEntityClone,
            OptionEntityLinearPatternTransform,
            OptionEntityLinearPattern,
            OptionEntityCircularPattern,
            OptionEntityMakeHelix,
            OptionEntityMakeHelixFromParams,
            OptionEntityMakeHelixFromEdge,
            OptionEntityMirror,
            OptionEntityMirrorAcrossEdge,
            OptionSelectWithPoint,
            OptionSelectAdd,
            OptionSelectRemove,
            OptionSceneClearAll,
            OptionSelectReplace,
            OptionHighlightSetEntity,
            OptionHighlightSetEntities,
            OptionNewAnnotation,
            OptionUpdateAnnotation,
            OptionEdgeLinesVisible,
            OptionObjectVisible,
            OptionObjectBringToFront,
            OptionObjectSetMaterialParamsPbr,
            OptionGetEntityType,
            OptionSolid3DGetAllEdgeFaces,
            OptionSolid2DAddHole,
            OptionSolid3DGetAllOppositeEdges,
            OptionSolid3DGetOppositeEdge,
            OptionSolid3DGetNextAdjacentEdge,
            OptionSolid3DGetPrevAdjacentEdge,
            OptionSolid3DGetCommonEdge,
            OptionSolid3DFilletEdge,
            OptionFaceIsPlanar,
            OptionFaceGetPosition,
            OptionFaceGetCenter,
            OptionFaceGetGradient,
            OptionSendObject,
            OptionEntitySetOpacity,
            OptionEntityFade,
            OptionMakePlane,
            OptionPlaneSetColor,
            OptionSetTool,
            OptionMouseMove,
            OptionMouseClick,
            OptionSketchModeDisable,
            OptionGetSketchModePlane,
            OptionCurveSetConstraint,
            OptionEnableSketchMode,
            OptionEnableDryRun,
            OptionDisableDryRun,
            OptionSetBackgroundColor,
            OptionSetCurrentToolProperties,
            OptionSetDefaultSystemProperties,
            OptionCurveGetType,
            OptionCurveGetControlPoints,
            OptionProjectEntityToPlane,
            OptionProjectPointsToPlane,
            OptionTakeSnapshot,
            OptionMakeAxesGizmo,
            OptionPathGetInfo,
            OptionPathGetCurveUuidsForVertices,
            OptionPathGetCurveUuid,
            OptionPathGetVertexUuids,
            OptionPathGetSketchTargetUuid,
            OptionHandleMouseDragStart,
            OptionHandleMouseDragMove,
            OptionHandleMouseDragEnd,
            OptionRemoveSceneObjects,
            OptionPlaneIntersectAndProject,
            OptionCurveGetEndPoints,
            OptionReconfigureStream,
            OptionImportFiles,
            OptionSetSceneUnits,
            OptionMass,
            OptionDensity,
            OptionVolume,
            OptionCenterOfMass,
            OptionSurfaceArea,
            OptionDefaultCameraFocusOn,
            OptionSetSelectionType,
            OptionSetSelectionFilter,
            OptionDefaultCameraSetOrthographic,
            OptionDefaultCameraSetPerspective,
            OptionDefaultCameraCenterToSelection,
            OptionDefaultCameraCenterToScene,
            OptionZoomToFit,
            OptionOrientToFace,
            OptionViewIsometric,
            OptionSolid3DGetExtrusionFaceInfo,
            OptionSelectClear,
            OptionSelectGet,
            OptionGetNumObjects,
            OptionSetObjectTransform,
            OptionBooleanUnion,
            OptionBooleanIntersection,
            OptionBooleanSubtract,
            OptionMakeOffsetPath,
            OptionAddHoleFromOffset,
            OptionSetGridReferencePlane,
        ],
        Field(discriminator="type"),
    ]
]



from pydantic import BaseModel, ConfigDict


class SideFace(BaseModel):
    """IDs for a side face, extruded from the path of some sketch/2D shape."""

    face_id: str

    path_id: str

    model_config = ConfigDict(protected_namespaces=())



"""Contains methods for accessing the api_tokens API paths: API tokens allow users to call the API outside of their session token that is used as a cookie in the user interface. Users can create, delete, and list their API tokens. But, of course, you need an API token to do this, so first be sure to generate one in the account UI."""  # noqa: E501



from pydantic import BaseModel, ConfigDict

from .base64data import Base64Data


class ExportFile(BaseModel):
    """A file to be exported to the client."""

    contents: Base64Data

    name: str

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class ErrorCode(str, Enum):
    """The type of error sent by the KittyCAD API."""  # noqa: E501

    """# Graphics engine failed to complete request, consider retrying """  # noqa: E501
    INTERNAL_ENGINE = "internal_engine"
    """# API failed to complete request, consider retrying """  # noqa: E501
    INTERNAL_API = "internal_api"
    """# User requested something geometrically or graphically impossible. Don't retry this request, as it's inherently impossible. Instead, read the error message and change your request. """  # noqa: E501
    BAD_REQUEST = "bad_request"
    """# Auth token is missing from the request """  # noqa: E501
    AUTH_TOKEN_MISSING = "auth_token_missing"
    """# Auth token is invalid in some way (expired, incorrect format, etc) """  # noqa: E501
    AUTH_TOKEN_INVALID = "auth_token_invalid"
    """# Client sent invalid JSON. """  # noqa: E501
    INVALID_JSON = "invalid_json"
    """# Client sent invalid BSON. """  # noqa: E501
    INVALID_BSON = "invalid_bson"
    """# Client sent a message which is not accepted over this protocol. """  # noqa: E501
    WRONG_PROTOCOL = "wrong_protocol"
    """# Problem sending data between client and KittyCAD API. """  # noqa: E501
    CONNECTION_PROBLEM = "connection_problem"
    """# Client sent a Websocket message type which the KittyCAD API does not handle. """  # noqa: E501
    MESSAGE_TYPE_NOT_ACCEPTED = "message_type_not_accepted"
    """# Client sent a Websocket message intended for WebRTC but it was configured as a WebRTC connection. """  # noqa: E501
    MESSAGE_TYPE_NOT_ACCEPTED_FOR_WEB_R_T_C = "message_type_not_accepted_for_web_r_t_c"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitEnergy(str, Enum):
    """The valid types of energy units."""  # noqa: E501

    """# British Thermal Unit (BTU) <https://en.wikipedia.org/wiki/British_thermal_unit> """  # noqa: E501
    BTU = "btu"
    """# Electron Volts (eV) <https://en.wikipedia.org/wiki/Electronvolt> """  # noqa: E501
    ELECTRONVOLTS = "electronvolts"
    """# Joules (or watt-seconds) <https://en.wikipedia.org/wiki/Joule> """  # noqa: E501
    JOULES = "joules"
    """# Kilocalories (often just called calories) <https://en.wikipedia.org/wiki/Kilocalorie> """  # noqa: E501
    KILOCALORIES = "kilocalories"
    """# Kilowatt hours (kWh) <https://en.wikipedia.org/wiki/Kilowatt-hour> """  # noqa: E501
    KILOWATT_HOURS = "kilowatt_hours"
    """# Watt hours (Wh) <https://en.wikipedia.org/wiki/Kilowatt-hour> """  # noqa: E501
    WATT_HOURS = "watt_hours"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class GetSketchModePlane(BaseModel):
    """The plane for sketch mode."""

    origin: Point3d

    x_axis: Point3d

    y_axis: Point3d

    z_axis: Point3d

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.camera_view_state import CameraViewState


class DefaultCameraGetView(BaseModel):
    """The response from the `DefaultCameraGetView` command."""

    view: CameraViewState

    model_config = ConfigDict(protected_namespaces=())
